package org.radargun.stressors;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.radargun.config.Stressor;
import org.radargun.features.KeyGeneratorAware;

/**
 * Produces a key generator which is retrieved from KeyGenerator-aware cache wrapper. This key generator
 * is then used to generate keys instead of the default key generator.
 */
@Stressor(doc = "Executes put and get operations with keys generated by cache-specific key generator.")
public class CacheSpecificKeyGenStressor extends StressTestStressor {

   private static final Log log = LogFactory.getLog(CacheSpecificKeyGenStressor.class);

   @Override
   public KeyGenerator getKeyGenerator() {
      synchronized(this) {
         if (keyGenerator == null) {
            if (cacheWrapper instanceof KeyGeneratorAware) {
               keyGenerator = ((KeyGeneratorAware)cacheWrapper).getKeyGenerator(getNumThreads() * getNumEntries());
            } else {
               throw new RuntimeException("Cache wrapper is not KeyGeneratorAware");
            }
         }
      }
      return keyGenerator;
   }

}
